#commons_trends_graphs.py

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import datetime as dt
import gc

hurricanes =['hermine','matthew','harvey', 'irma','florence','michael','dorian']
plt.rcParams.update({'font.size': 12})

for h in hurricanes:
	df = pd.read_csv(h+'/full_yearly_'+h+'_cf.csv')
	df.sort_values(by=['year','month'], inplace = True)
	df['month-year'] = df['month'].astype(str) + '-' + df['year'].astype(str)
	df['log_light'] = np.log(df['light'])

	fig, (ax1, ax2) = plt.subplots(nrows=1, ncols=2, sharey=True)
	fig.suptitle(h.capitalize()+': Combined Aid')
	sns.lineplot(ax=ax1, data=df, x='month-year', y='log_light', hue='fema_treat')
	ax1.set_title('Monthly index')
	ax1.axvline(x='9-2017', color='green')
	ax1.set_xticks(range(0, len(df),4))
	ax1.set_ylabel('Radiance')
	ax1.legend(labels=['control','treatment'], loc='best')
	sns.lineplot(ax=ax2,data=df, x='year', y='log_light', hue='fema_treat')
	ax2.set_title('Yearly index')
	ax2.axvline(x=2017.75, color='green')
	ax2.legend(labels=['control','treatment'], loc='best')
	plt.tight_layout()
	plt.savefig('commontrend_'+h+'.png')
	print (h+ 'done! #1')
	plt.close()

	fig, (ax3, ax4) = plt.subplots(nrows=1, ncols=2, sharey=True)
	fig.suptitle(h.capitalize()+': Individual Aid')
	sns.lineplot(ax=ax3, data=df, x='month-year', y='log_light', hue='fema_IA')
	ax3.set_title('Monthly index')
	ax3.axvline(x='9-2017', color='green')
	ax3.set_xticks(range(0, len(df),4))
	ax3.set_ylabel('Radiance')
	ax3.legend(labels=['control','treatment'], loc='best')
	sns.lineplot(ax=ax4,data=df, x='year', y='log_light', hue='fema_IA')
	ax4.set_title('Yearly index')
	ax4.axvline(x=2017.75, color='green')
	ax4.legend(labels=['control','treatment'], loc='best')
	plt.tight_layout()
	plt.savefig('commontrend_'+h+'_ia.png')
	plt.close()
	print (h+ 'done! #2')
	del df, fig, ax1, ax2, ax3, ax4
	gc.collect()
