#linearizer.py
#linearlize each year & concatonate, add fema and imdat vars
import pandas as pd
import numpy as np
import geopandas as gpd
import glob
from shapely.geometry import Point, Polygon, LineString
import calendar
from numpy import inf
import datetime

months = ['Aug','Nov','Jul','Dec','Jun','Mar','Sep','Jan','Feb','Apr','May','Oct']

df_list = []
for file in glob.glob('geofiles/*_monthly_hermine_cf_geo.shp'):
    gdf = gpd.read_file(file)
    print (gdf.columns)
    gdf.drop('index_righ', axis=1, inplace=True)
    new_df = gdf[months].unstack().reset_index()
    new_df.drop('level_1', axis =1, inplace=True)
    new_df.rename({'level_0':'month', 0:'light'}, axis =1, inplace=True)
    new_df['geometry'] = np.tile(gdf.geometry, 12)
    new_df['county'] = np.tile(gdf.FIPS, 12)
    new_df['year'] = np.tile(gdf.year, 12)
    df_list.append(new_df)
big_df = pd.concat(df_list)
big_df.sort_values(by=['year'], inplace = True)

#bound the lights between 0 and 50
big_df = big_df[(big_df['light']>=0) & (big_df['light']<50)]
big_df.reset_index(inplace = True)

#chanage format of month & save
month_dict = {month: index for index, month in enumerate(calendar.month_abbr) if month}
big_df['month'] = big_df.month.apply(lambda x: month_dict[x])
big_df['month'] = big_df['month'].astype(int)
big_df['year'] = big_df['year'].astype(int)
big_df['post'] = np.where(((big_df.year == 2016) & (big_df.month >= 9)) | (big_df.year > 2016), 1, 0)
big_df.to_csv('yearly_hermine_cf.csv', index = False)

#create FEMA treatment var & save
fema = pd.read_csv('fema_hermine.csv', sep=';').fillna(0)
assistance = ['Individual Assistance','Public Assistance','PA-A','PA-B','PA-C','PA-D','PA-E','PA-F','PA-G','PA-H']
ab_ass = ['PA-A', 'PA-B']
fema['fema_tr'] = fema[assistance].sum(axis=1)
fema['fema_pub_ab'] = fema[ab_ass].sum(axis=1)
fema['fema_pub'] = fema['fema_tr'] - fema['Individual Assistance']
fema['fema_treat'] = np.where(fema.fema_tr>0, 1, 0)
fema['county'] = fema.FIPS
fema.rename({'Individual Assistance':'fema_IA'}, axis =1, inplace=True)
fema = fema[['county','fema_treat','fema_tr','fema_pub_ab', 'fema_pub', 'fema_IA']]
fema.fillna(0, inplace=True)
df1 = big_df.merge(fema, left_on= 'county', right_on = 'county')
#df1.to_csv('fema_yearly_irma_cf.csv', index = False)

df1['fema_int'] = df1['fema_treat']*df1['post']
df1['fema_int_ia'] = df1['fema_IA']*df1['post']
#df2['imdat_int'] = df2['imdat_treat']*df2['post']
df1['log_light'] = np.log(df1['light'])
df1.log_light.replace(-inf, np.log(0.0000000000001), inplace =True)

auxdat = pd.read_csv('/p/projects/impactee/kelsey/night_light_data/viirs_monthly/monthly/NCHSURCodes2013.csv')
full_df = df1.merge(auxdat, left_on= 'county', right_on = 'FIPS code')
#full_df.drop(['FIPS','county_name','county_code'], axis=1, inplace=True)
full_df.columns = [c.replace(' ', '_') for c in full_df.columns]
full_df.rename({'2013_code':'urban'}, axis =1, inplace=True)
print (full_df.columns)
full_df.to_csv('full_yearly_hermine_cf.csv', index = False)


