import pandas as pd
import numpy as np
import statsmodels.formula.api as smf
import gc
import math
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
import glob
import joblib
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import RandomizedSearchCV
import xgboost as xgb
from sklearn.metrics import r2_score, mean_squared_error
import shap
import matplotlib.pyplot as plt
import seaborn as sns
from lightgbm import LGBMClassifier
from varname import nameof
from eli5.sklearn import PermutationImportance
from sklearn import metrics

def print_metrics(y_test, y_preds):
    acc = metrics.accuracy_score(y_test, y_preds)
    print ('ACCURACY: ', acc)
    conf_mat = metrics.confusion_matrix(y_test, y_preds)
    print ('CONFUSION MATRIX')
    print (conf_mat)
    class_rpt = metrics.classification_report(y_test, y_preds)

def var_maker(dataframe, datedict1, datedict2):
    dataframe['base_n'] = dataframe['hurricane'].map(datedict1)
    dataframe['base_ia'] = dataframe['hurricane'].map(datedict2)
    
    dataframe['b_off'] = dataframe.apply(lambda row: np.where((row['base_n']<0), 1, 0), axis=1) 
    dataframe['b_off'] = dataframe.apply(lambda row: np.where((row['base_n']<0) & (row['coi']<0) & (row['coi']<row['base_n']), 1, 0), axis=1) 
    dataframe['b_off'] = dataframe.b_off.astype(int)
    
    #dataframe['keep'] = dataframe.apply(lambda row: np.where((row['base_n']<0) & (row['coi']>0), 1, 0), axis=1) #correct
    #dataframe['keep'] = dataframe.apply(lambda row: np.where((row['base_n']<0) & (row['coi']>row['base_n']), 1, row['keep']), axis=1) # correct 
    #dataframe['keep'] = dataframe.apply(lambda row: np.where((row['base_n']>0) & (row['coi']>row['base_n']), 1, row['keep']), axis=1)   #delete 
    
    dataframe['b_off_ia'] = dataframe.apply(lambda row: np.where((row['base_ia']<0), 1, 0), axis=1) 
    dataframe['b_off_ia'] = dataframe.apply(lambda row: np.where((row['base_ia']<0) & (row['coi_ia']<0) & (row['coi_ia']<row['base_ia']), 1, 0), axis=1) 
    dataframe['b_off_ia'] = dataframe.b_off_ia.astype(int)
    return dataframe

def build_algos(dataframe):
    prelim = dataframe[['fema_treat','fema_IA','b_off','b_off_ia', 'urban', 'per_female', 'per_white', 'per_black', 'per_aa',
           'other_race', 'u20', 'middle_age', 'elderly', 'employment',
           'population', 'income','individual_cost', 'individual_apps', 'total_cost']]
    prelim = prelim.drop_duplicates()

    treat = prelim[prelim['fema_treat']==1] 
    treat_ia = prelim[prelim['fema_IA']==1] 

    Xmat = treat[['urban', 'per_female', 'per_white', 'per_black', 'per_aa',
           'other_race', 'u20', 'middle_age', 'elderly', 'employment',
           'population', 'income','individual_cost', 'individual_apps', 'total_cost']]
    yvec = treat['b_off']
    print ('data loaded...')
    Xtrain, Xtest, ytrain, ytest = train_test_split(Xmat, yvec, test_size = .3, random_state = 0)
    print ('test/train split created')

    print ('NORMAL AID')
    rf = RandomForestClassifier(random_state = 0)   
    rf.fit(Xtrain, ytrain)
    print ('Random Forest Classifier Results')
    print ('Random Forest TRAIN METRICS')
    print_metrics(ytrain, rf.predict(Xtrain))
    print ('Random Forest  TEST METRICS')
    print_metrics(ytest, rf.predict(Xtest))

    xgbr = xgb.XGBClassifier() 
    xgbr.fit(Xtrain, ytrain)
    print ('XGBoost Classifier Results')
    print ('XGBoost TRAIN METRICS')
    print_metrics(ytrain, xgbr.predict(Xtrain))
    print ('XGBoost  TEST METRICS')
    print_metrics(ytest, xgbr.predict(Xtest))

    lgbm = LGBMClassifier(random_state = 0, objective='binary')     
    lgbm.fit(Xtrain, ytrain)
    print ('Light GBM Classifier Results')
    print ('Light GBM TRAIN METRICS')
    print_metrics(ytrain, lgbm.predict(Xtrain))
    print ('Light GBM TEST METRICS')
    print_metrics(ytest, lgbm.predict(Xtest))

    print ('INDIVIDUAL AID')
    Xmat = treat_ia[['urban', 'per_female', 'per_white', 'per_black', 'per_aa',
           'other_race', 'u20', 'middle_age', 'elderly', 'employment',
           'population', 'income','individual_cost', 'individual_apps', 'total_cost']]
    yvec = treat_ia['b_off_ia']
    print ('data loaded...')
    Xtrain, Xtest, ytrain, ytest = train_test_split(Xmat, yvec, test_size = .3, random_state = 0)
    print ('test/train split created')

    print ('INDIVIDUAL ASSISTANCE')
    rf = RandomForestClassifier(random_state = 0)   
    rf.fit(Xtrain, ytrain)
    print ('Random Forest Classifier Results')
    print ('Random Forest TRAIN METRICS')
    print_metrics(ytrain, rf.predict(Xtrain))
    print ('Random Forest  TEST METRICS')
    print_metrics(ytest, rf.predict(Xtest))

    xgbr = xgb.XGBClassifier() 
    xgbr.fit(Xtrain, ytrain)
    print ('XGBoost Classifier Results')
    print ('XGBoost TRAIN METRICS')
    print_metrics(ytrain, xgbr.predict(Xtrain))
    print ('XGBoost  TEST METRICS')
    print_metrics(ytest, xgbr.predict(Xtest))

    lgbm = LGBMClassifier(random_state = 0, objective='binary')     
    lgbm.fit(Xtrain, ytrain)
    print ('Light GBM Classifier Results')
    print ('Light GBM TRAIN METRICS')
    print_metrics(ytrain, lgbm.predict(Xtrain))
    print ('Light GBM TEST METRICS')
    print_metrics(ytest, lgbm.predict(Xtest))
    print ('\n')
    print ('\n')
    return


def run_shaps(dataframe, time):
    prelim = dataframe[['fema_treat','fema_IA','b_off','b_off_ia','urban', 'per_female', 'per_white', 'per_black', 'per_aa',
           'other_race', 'u20', 'middle_age', 'elderly', 'employment',
           'population', 'income', 'individual_cost', 'individual_apps', 'total_cost']]
    prelim = prelim.drop_duplicates()

    treat = prelim[prelim['fema_treat']==1] 
    treat_ia = prelim[prelim['fema_IA']==1] 
    Xmat = treat[['urban', 'per_female', 'per_white', 'per_black', 'per_aa',
           'other_race', 'u20', 'middle_age', 'elderly', 'employment',
           'population', 'income', 'individual_cost', 'individual_apps', 'total_cost']]
    yvec = treat['b_off']
    Xmat_ia = treat_ia[['urban', 'per_female', 'per_white', 'per_black', 'per_aa',
           'other_race', 'u20', 'middle_age', 'elderly', 'employment',
           'population', 'income', 'individual_cost', 'individual_apps', 'total_cost']]
    yvec_ia = treat_ia['b_off_ia'] 
    print ('data loaded...')
    Xtrain, Xtest, ytrain, ytest = train_test_split(Xmat, yvec, test_size = .3, random_state = 0)
    Xtrain_ia, Xtest_ia, ytrain_ia, ytest_ia = train_test_split(Xmat_ia, yvec_ia, test_size = .3, random_state = 0)
    print ('test/train splits created')
    if time == 'dfo':
        print ('working on dfo')
        model = LGBMClassifier(random_state = 0, objective='binary')  
        model.fit(Xtrain, ytrain)
        print_metrics(ytest, model.predict(Xtest))
        #individual aid
        model_ia = LGBMClassifier(random_state = 0, objective='binary')  
        model_ia.fit(Xtrain_ia, ytrain_ia)
        print_metrics(ytest_ia, model_ia.predict(Xtest_ia))
        print ('\n')
        shap = make_shap_df(model, model_ia, Xmat, Xmat_ia)
        perm_n = PermutationImportance(model,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest, ytest)
        perm_ia = PermutationImportance(model_ia,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest_ia, ytest_ia)
        perm = pd.DataFrame(zip(perm_n.feature_importances_, Xtest.columns), columns = ['perm_n','feat'])
        perm['perm_ia'] = perm_ia.feature_importances_
        perm['norm'] = (np.abs(perm['perm_n'])/sum(np.abs(perm['perm_n']))*100)
        perm['norm_ia'] = (np.abs(perm['perm_ia'])/sum(np.abs(perm['perm_ia']))*100)
        return shap, perm
    if time == 'df3':
        model = RandomForestClassifier(random_state = 0)
        model.fit(Xtrain, ytrain)
        print_metrics(ytest, model.predict(Xtest))
        #individual aid
        model_ia = xgb.XGBClassifier()
        model_ia.fit(Xtrain_ia, ytrain_ia)
        print_metrics(ytest_ia, model_ia.predict(Xtest_ia))
        print ('\n')
        shap = make_shap_df(model, model_ia, Xmat, Xmat_ia)
        perm_n = PermutationImportance(model,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest, ytest)
        perm_ia = PermutationImportance(model_ia,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest_ia, ytest_ia)
        perm = pd.DataFrame(zip(perm_n.feature_importances_, Xtest.columns), columns = ['perm_n','feat'])
        perm['perm_ia'] = perm_ia.feature_importances_
        perm['norm'] = (np.abs(perm['perm_n'])/sum(np.abs(perm['perm_n']))*100)
        perm['norm_ia'] = (np.abs(perm['perm_ia'])/sum(np.abs(perm['perm_ia']))*100)
        return shap, perm
    if time == 'df6':
        model = xgb.XGBClassifier()
        model.fit(Xtrain, ytrain)
        print_metrics(ytest, model.predict(Xtest))
        #individual aid
        model_ia = LGBMClassifier(random_state = 0, objective='binary') 
        model_ia.fit(Xtrain_ia, ytrain_ia)
        print_metrics(ytest_ia, model_ia.predict(Xtest_ia))
        print ('\n')
        shap = make_shap_df(model, model_ia, Xmat, Xmat_ia)
        perm_n = PermutationImportance(model,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest, ytest)
        perm_ia = PermutationImportance(model_ia,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest_ia, ytest_ia)
        perm = pd.DataFrame(zip(perm_n.feature_importances_, Xtest.columns), columns = ['perm_n','feat'])
        perm['perm_ia'] = perm_ia.feature_importances_
        perm['norm'] = (np.abs(perm['perm_n'])/sum(np.abs(perm['perm_n']))*100)
        perm['norm_ia'] = (np.abs(perm['perm_ia'])/sum(np.abs(perm['perm_ia']))*100)
        return shap, perm
    if time == 'df12':
        model = LGBMClassifier(random_state = 0, objective='binary') 
        model.fit(Xtrain, ytrain)
        print_metrics(ytest, model.predict(Xtest))
        #individual aid
        model_ia = xgb.XGBClassifier()
        model_ia.fit(Xtrain_ia, ytrain_ia)
        print_metrics(ytest_ia, model_ia.predict(Xtest_ia))
        print ('\n')
        shap = make_shap_df(model, model_ia, Xmat, Xmat_ia)
        perm_n = PermutationImportance(model,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest, ytest)
        perm_ia = PermutationImportance(model_ia,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest_ia, ytest_ia)
        perm = pd.DataFrame(zip(perm_n.feature_importances_, Xtest.columns), columns = ['perm_n','feat'])
        perm['perm_ia'] = perm_ia.feature_importances_
        perm['norm'] = (np.abs(perm['perm_n'])/sum(np.abs(perm['perm_n']))*100)
        perm['norm_ia'] = (np.abs(perm['perm_ia'])/sum(np.abs(perm['perm_ia']))*100)
        return shap, perm

def run_nofema_shaps(dataframe, time):
    prelim = dataframe[['fema_treat','fema_IA','b_off','b_off_ia','urban', 'per_female', 'per_white', 'per_black', 'per_aa',
           'other_race', 'u20', 'middle_age', 'elderly', 'employment',
           'population', 'income']]
    prelim = prelim.drop_duplicates()

    treat = prelim[prelim['fema_treat']==1] 
    treat_ia = prelim[prelim['fema_IA']==1] 
    Xmat = treat[['urban', 'per_female', 'per_white', 'per_black', 'per_aa',
           'other_race', 'u20', 'middle_age', 'elderly', 'employment',
           'population', 'income']]
    yvec = treat['b_off']
    Xmat_ia = treat_ia[['urban', 'per_female', 'per_white', 'per_black', 'per_aa',
           'other_race', 'u20', 'middle_age', 'elderly', 'employment',
           'population', 'income']]
    yvec_ia = treat_ia['b_off_ia'] 
    print ('data loaded...')
    Xtrain, Xtest, ytrain, ytest = train_test_split(Xmat, yvec, test_size = .3, random_state = 0)
    Xtrain_ia, Xtest_ia, ytrain_ia, ytest_ia = train_test_split(Xmat_ia, yvec_ia, test_size = .3, random_state = 0)
    print ('test/train splits created')
    if time == 'dfo':
        print ('working on dfo')
        model = RandomForestClassifier(random_state = 0) 
        model.fit(Xtrain, ytrain)
        print_metrics(ytest, model.predict(Xtest))
        #individual aid
        model_ia = LGBMClassifier(random_state = 0, objective='binary')   
        model_ia.fit(Xtrain_ia, ytrain_ia)
        print_metrics(ytest_ia, model_ia.predict(Xtest_ia))
        print ('\n')
        shap = make_shap_df(model, model_ia, Xmat, Xmat_ia)
        perm_n = PermutationImportance(model,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest, ytest)
        perm_ia = PermutationImportance(model_ia,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest_ia, ytest_ia)
        perm = pd.DataFrame(zip(perm_n.feature_importances_, Xtest.columns), columns = ['perm_n','feat'])
        perm['perm_ia'] = perm_ia.feature_importances_
        perm['norm'] = (np.abs(perm['perm_n'])/sum(np.abs(perm['perm_n']))*100)
        perm['norm_ia'] = (np.abs(perm['perm_ia'])/sum(np.abs(perm['perm_ia']))*100)
        return shap, perm
    if time == 'df3':
        model = RandomForestClassifier(random_state = 0)  
        model.fit(Xtrain, ytrain)
        print_metrics(ytest, model.predict(Xtest))
        #individual aid
        model_ia = LGBMClassifier(random_state = 0, objective='binary')     
        model_ia.fit(Xtrain_ia, ytrain_ia)
        print_metrics(ytest_ia, model_ia.predict(Xtest_ia))
        print ('\n')
        shap = make_shap_df(model, model_ia, Xmat, Xmat_ia)
        perm_n = PermutationImportance(model,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest, ytest)
        perm_ia = PermutationImportance(model_ia,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest_ia, ytest_ia)
        perm = pd.DataFrame(zip(perm_n.feature_importances_, Xtest.columns), columns = ['perm_n','feat'])
        perm['perm_ia'] = perm_ia.feature_importances_
        perm['norm'] = (np.abs(perm['perm_n'])/sum(np.abs(perm['perm_n']))*100)
        perm['norm_ia'] = (np.abs(perm['perm_ia'])/sum(np.abs(perm['perm_ia']))*100)
        return shap, perm
    if time == 'df6':
        model = RandomForestClassifier(random_state = 0)     
        model.fit(Xtrain, ytrain)
        print_metrics(ytest, model.predict(Xtest))
        #individual aid
        model_ia = LGBMClassifier(random_state = 0, objective='binary')
        model_ia.fit(Xtrain_ia, ytrain_ia)
        print_metrics(ytest_ia, model_ia.predict(Xtest_ia))
        print ('\n')
        shap = make_shap_df(model, model_ia, Xmat, Xmat_ia)
        perm_n = PermutationImportance(model,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest, ytest)
        perm_ia = PermutationImportance(model_ia,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest_ia, ytest_ia)
        perm = pd.DataFrame(zip(perm_n.feature_importances_, Xtest.columns), columns = ['perm_n','feat'])
        perm['perm_ia'] = perm_ia.feature_importances_
        perm['norm'] = (np.abs(perm['perm_n'])/sum(np.abs(perm['perm_n']))*100)
        perm['norm_ia'] = (np.abs(perm['perm_ia'])/sum(np.abs(perm['perm_ia']))*100)
        return shap, perm
    if time == 'df12':
        model = RandomForestClassifier(random_state = 0)   
        model.fit(Xtrain, ytrain)
        print_metrics(ytest, model.predict(Xtest))
        #individual aid
        model_ia = LGBMClassifier(random_state = 0, objective='binary')    
        model_ia.fit(Xtrain_ia, ytrain_ia)
        print_metrics(ytest_ia, model_ia.predict(Xtest_ia))
        print ('\n')
        shap = make_shap_df(model, model_ia, Xmat, Xmat_ia)
        perm_n = PermutationImportance(model,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest, ytest)
        perm_ia = PermutationImportance(model_ia,cv='prefit', n_iter = 50, scoring='neg_root_mean_squared_error' ).fit(Xtest_ia, ytest_ia)
        perm = pd.DataFrame(zip(perm_n.feature_importances_, Xtest.columns), columns = ['perm_n','feat'])
        perm['perm_ia'] = perm_ia.feature_importances_
        perm['norm'] = (np.abs(perm['perm_n'])/sum(np.abs(perm['perm_n']))*100)
        perm['norm_ia'] = (np.abs(perm['perm_ia'])/sum(np.abs(perm['perm_ia']))*100)
        return shap, perm

def make_shap_df(model_NORMAL, model_IA, Xmat_NORMAL, Xmat_IA):
    model_shap = shap.TreeExplainer(model_NORMAL)
    if str(model_NORMAL)[0] == 'X':
        values = model_shap.shap_values(Xmat_NORMAL)
    else: 
        values = (model_shap.shap_values(Xmat_NORMAL))[1]
    shap_v = pd.DataFrame(values)
    feature_list = Xmat_NORMAL.columns.to_list()
    shap_v.columns = feature_list
    k = pd.DataFrame(shap_v.mean()).reset_index()
    k.columns = ['Variable','SHAP_mean']
    k['SHAP_abs'] = np.abs(k['SHAP_mean'])
    k['SHAP_norm'] = (k['SHAP_abs']/sum(np.abs(k['SHAP_mean']))*100)
    k['SHAP_dir'] = (k['SHAP_mean']/sum(np.abs(k['SHAP_mean']))*100)
    model_shap_ia = shap.TreeExplainer(model_IA)
    if str(model_IA)[0] == 'X':
        values = model_shap_ia.shap_values(Xmat_IA)
    else:
        values = (model_shap_ia.shap_values(Xmat_IA))[1]
    shap_v_ia = pd.DataFrame(values)
    k['SHAP_mean_ia'] = shap_v_ia.mean().reset_index(drop=True)
    k['SHAP_abs_ia'] = np.abs(k['SHAP_mean_ia'])
    k['SHAP_norm_ia'] = (k['SHAP_abs_ia']/sum(np.abs(k['SHAP_mean_ia']))*100)
    k['SHAP_dir_ia'] = (k['SHAP_mean_ia']/sum(np.abs(k['SHAP_mean_ia']))*100)
    return k


#read in all vars & distill from so many time chunks
full = pd.read_csv('Desktop/Desktop/correct_full_final_fin_nodupe.csv')
varss = full[['fema_treat','fema_IA','urban', 'per_female', 'per_white', 'per_black', 'per_aa','other_race', 'u20', 'middle_age', 'elderly', 'employment','population', 'income', 'individual_cost', 'individual_apps', 'total_cost', 'hurricane','county']]
varss = varss.drop_duplicates()

#overall
diff_ndf = pd.read_csv('Desktop/Desktop/LOO_normal_nodupes.csv')
diff_iadf = pd.read_csv('Desktop/Desktop/LOO_ia_nodupes.csv')
diff_df = pd.merge(diff_ndf, diff_iadf, left_on =['hurricane','county'], right_on =['hurricane','county'])
dfo = pd.merge(diff_df, varss, left_on =['hurricane','county'], right_on =['hurricane','county'])
normal_dict = {'hermine': -0.029264,'matthew': -0.121274,'harvey': -0.018460,'irma':0.0,'florence': 0.009010,'michael': 0.034283,'dorian': -0.098285}
ia_dict = {'hermine': -4.034148e-02,'matthew': -1.474698e-01,'harvey': -1.575077e-02,'irma': -1.186785e-01,'florence': 3.616383e-02,'michael': 7.017945e-02, 'dorian':0.0}

dfo = var_maker(dfo, normal_dict, ia_dict)

#3 month
diff_ndf_3m = pd.read_csv('Desktop/Desktop/LOO_normal_3m_nodupes_local.csv')
diff_iadf_3m = pd.read_csv('Desktop/Desktop/LOO_ia_3m_nodupes_local.csv')
diff_df_3m = pd.merge(diff_ndf_3m, diff_iadf_3m, left_on =['hurricane','county'], right_on =['hurricane','county'])
df3 = pd.merge(diff_df_3m, varss, left_on =['hurricane','county'], right_on =['hurricane','county'])
dict_3m = {'hermine': -0.037513,'matthew': -0.294144,'harvey': -0.076423,'irma':0.0,'florence': -115.202699,'michael': 0.071191,'dorian': -0.308683}
ia_dict_3m = {'hermine': 2.145479e-02,'matthew': -3.215332e-01,'harvey': 4.111515e-03,'irma': -1.102184e-01,'florence':-6.120435e+00,'michael': 2.154394e-01, 'dorian':0.0}

df3 = var_maker(df3, dict_3m, ia_dict_3m)

#6 month
diff_ndf_6m = pd.read_csv('Desktop/Desktop/LOO_normal_6m_nodupes_local.csv')
diff_iadf_6m = pd.read_csv('Desktop/Desktop/LOO_ia_6m_nodupes_local.csv')
diff_df_6m = pd.merge(diff_ndf_6m, diff_iadf_6m, left_on =['hurricane','county'], right_on =['hurricane','county'])
df6 = pd.merge(diff_df_6m, varss, left_on =['hurricane','county'], right_on =['hurricane','county'])
dict_6m = {'hermine':0.029559,'matthew': -0.207998,'harvey': -0.010603,'irma':0.0,'florence': 239.277902,'michael': 0.031663,'dorian': 0.011163}
ia_dict_6m = {'hermine': 1.011028e-01,'matthew': -2.325685e-01,'harvey': 1.052636e-03,'irma': -1.692363e-02,'florence': 1.241459e+03,'michael': 1.609224e-01, 'dorian':0.0}

df6 = var_maker(df6, dict_6m, ia_dict_6m)

# 12 month
diff_ndf_12m = pd.read_csv('Desktop/Desktop/LOO_normal_1year_nodupes_local.csv')
diff_iadf_12m = pd.read_csv('Desktop/Desktop/LOO_ia_1year_nodupes_local.csv')
diff_df_12m = pd.merge(diff_ndf_12m, diff_iadf_12m, left_on =['hurricane','county'], right_on =['hurricane','county'])
df12 = pd.merge(diff_df_12m, varss, left_on =['hurricane','county'], right_on =['hurricane','county'])
dict_1y = {'hermine': -0.012319,'matthew': -0.040512,'harvey': -0.002309,'irma':0.0,'florence': 0.072602,'michael': -0.018790,'dorian': 0.131694}
ia_dict_1y = {'hermine': -8.305838e-02,'matthew': -1.657053e-01,'harvey': 1.945521e-02,'irma': -6.792869e-02,'florence': 7.306383e-02,'michael': -4.971115e-02, 'dorian':0.0}

df12 = var_maker(df12, dict_1y, ia_dict_1y)


#for BETTER OFF
print ('TESTING ALGOS')
build_algos(dfo)        #lgbm, lgbm             overall
build_algos(df3)        #rf, xg                 3 months
build_algos(df6)        #xg, lgbm               6 months
build_algos(df12)       #lgbm, xg               12 months

print ('TESTING ALGOS without fema vars')       #make changes here
build_algos(dfo)        #rf, lgbm
build_algos(df3)        #rf, lgbm
build_algos(df6)        #rf, lgbm
build_algos(df12)       #rf, lgbm

ko, po = run_shaps(dfo, 'dfo')
k3, p3 = run_shaps(df3, 'df3')
k6, p6 = run_shaps(df6, 'df6')
k12, p12 = run_shaps(df12, 'df12')

ko.to_csv('Desktop/finals/better_off/datasets/ko.csv')
k3.to_csv('Desktop/finals/better_off/datasets/k3.csv')
k6.to_csv('Desktop/finals/better_off/datasets/k6.csv')
k12.to_csv('Desktop/finals/better_off/datasets/k12.csv')

po.to_csv('Desktop/finals/better_off/datasets/po.csv')
p3.to_csv('Desktop/finals/better_off/datasets/p3.csv')
p6.to_csv('Desktop/finals/better_off/datasets/p6.csv')
p12.to_csv('Desktop/finals/better_off/datasets/p12.csv')

ko, po = run_nofema_shaps(dfo, 'dfo')
k3, p3 = run_nofema_shaps(df3, 'df3')
k6, p6 = run_nofema_shaps(df6, 'df6')
k12, p12 = run_nofema_shaps(df12, 'df12')

palette = sns.color_palette('twilight_shifted')  #for "better off"
var_names = ['urban', '% female', '% white', '% black', '% asian', 'other race','young pop.','middle age','elderly','employment','population','income','individ. cost','individ. apps.','total cost']

#plot the proportion of better off counties over time
tot_counties = 1669
data =  [['3 months', (365/tot_counties)],['6 months', (239/tot_counties)],['12 months', (378/tot_counties)],['Full timeline',(422/tot_counties)]]
df_boff = pd.DataFrame(data, columns =['time', 'boff'])
df_boff['boff_per'] = df_boff['boff']*100
df_boff['boff_ia'] = pd.Series([334/tot_counties, 237/tot_counties, 405/tot_counties, 395/tot_counties])
df_boff['boff_ia_per'] = df_boff['boff_ia']*100

sns.set_theme()
sns.lineplot(data = df_boff, x ='time', y='boff_per', label = 'Any Aid')
sns.lineplot(data = df_boff, x ='time', y='boff_ia_per', label = 'Individual Aid')
plt.xlabel('Months from hurricane')
plt.ylabel('Percentage of better off counties')
plt.show()



#plot overall SHAP graph
sns.set_theme()
#plotting all together
fig, axs = plt.subplots(nrows=4, ncols=2, sharex =True)
fig.set_dpi(350)
#fig.suptitle('Normalized Mean SHAP Importances')
#sns.set(font_scale=.75)
sns.barplot(ax =axs[0,0], x = k3['SHAP_mean'], y = var_names, palette = palette, dodge=False).set_title('3 months')
sns.barplot(ax =axs[0,1], x = k3['SHAP_mean_ia'], y = var_names, palette = palette, dodge=False).set_title('3 months, IA')
sns.barplot(ax =axs[1,0], x = k6['SHAP_mean'], y = var_names, palette = palette, dodge=False).set_title('6 months')
sns.barplot(ax =axs[1,1], x = k6['SHAP_mean_ia'], y = var_names, palette = palette, dodge=False).set_title('6 months, IA')
sns.barplot(ax =axs[2,0], x = k12['SHAP_mean'], y = var_names, palette = palette, dodge=False).set_title('12 months')
sns.barplot(ax =axs[2,1], x = k12['SHAP_mean_ia'], y = var_names, palette = palette, dodge=False).set_title('12 months, IA')
sns.barplot(ax =axs[3,0], x = ko['SHAP_mean'], y = var_names, palette = palette, dodge=False).set_title('Overall')
sns.barplot(ax =axs[3,1], x = ko['SHAP_mean_ia'], y = var_names, palette = palette, dodge=False).set_title('Overall, IA')
#swap the two columnshere!!
axs[0,0].yaxis.set_label_position('right')
axs[0,0].yaxis.tick_right()
axs[1,0].yaxis.set_label_position('right')
axs[1,0].yaxis.tick_right()
axs[2,0].yaxis.set_label_position('right')
axs[2,0].yaxis.tick_right()
axs[3,0].yaxis.set_label_position('right')
axs[3,0].yaxis.tick_right()
axs[0,1].set(yticklabels=[])
axs[0,1].yaxis.tick_left()
axs[1,1].set(yticklabels=[])
axs[1,1].yaxis.tick_left()
axs[2,1].set(yticklabels=[])
axs[2,1].yaxis.tick_left()
axs[3,1].set(yticklabels=[])
axs[3,1].yaxis.tick_left()
axs[0,0].set_ylabel('')
axs[0,0].set_xlabel('')
axs[1,0].set_ylabel('')
axs[1,0].set_xlabel('')
axs[2,0].set_ylabel('')
axs[2,0].set_xlabel('')
axs[3,0].set_ylabel('')
axs[3,0].set_xlabel('SHAP importance, percentage points')
axs[0,1].set_ylabel('')
axs[0,1].set_xlabel('')
axs[1,1].set_ylabel('')
axs[1,1].set_xlabel('')
axs[2,1].set_ylabel('')
axs[2,1].set_xlabel('')
axs[3,1].set_ylabel('')
axs[3,1].set_xlabel('SHAP importance, percentage points')
plt.tight_layout()
plt.subplots_adjust(wspace=0.4,hspace=0.2)
plt.show()

#plot overall SHAP graph
sns.set_theme()
#plotting all together
fig, axs = plt.subplots(nrows=4, ncols=2, sharex =True)
fig.set_dpi(350)
fig.suptitle('Permutation Importances')
#sns.set(font_scale=.75)
sns.barplot(ax =axs[0,0], x = p3['norm'], y = var_names, palette = palette, dodge=False).set_title('3 months')
sns.barplot(ax =axs[0,1], x = p3['norm_ia'], y = var_names, palette = palette, dodge=False).set_title('3 months, IA')
sns.barplot(ax =axs[1,0], x = p6['norm'], y = var_names, palette = palette, dodge=False).set_title('6 months')
sns.barplot(ax =axs[1,1], x = p6['norm_ia'], y = var_names, palette = palette, dodge=False).set_title('6 months, IA')
sns.barplot(ax =axs[2,0], x = p12['norm'], y = var_names, palette = palette, dodge=False).set_title('12 months')
sns.barplot(ax =axs[2,1], x = p12['norm_ia'], y = var_names, palette = palette, dodge=False).set_title('12 months, IA')
sns.barplot(ax =axs[3,0], x = po['norm'], y = var_names, palette = palette, dodge=False).set_title('Overall')
sns.barplot(ax =axs[3,1], x = po['norm_ia'], y = var_names, palette = palette, dodge=False).set_title('Overall, IA')
#swap the two columnshere!!
axs[0,0].yaxis.set_label_position('right')
axs[0,0].yaxis.tick_right()
axs[1,0].yaxis.set_label_position('right')
axs[1,0].yaxis.tick_right()
axs[2,0].yaxis.set_label_position('right')
axs[2,0].yaxis.tick_right()
axs[3,0].yaxis.set_label_position('right')
axs[3,0].yaxis.tick_right()
axs[0,1].set(yticklabels=[])
axs[0,1].yaxis.tick_left()
axs[1,1].set(yticklabels=[])
axs[1,1].yaxis.tick_left()
axs[2,1].set(yticklabels=[])
axs[2,1].yaxis.tick_left()
axs[3,1].set(yticklabels=[])
axs[3,1].yaxis.tick_left()
axs[0,0].set_ylabel('')
axs[0,0].set_xlabel('')
axs[1,0].set_ylabel('')
axs[1,0].set_xlabel('')
axs[2,0].set_ylabel('')
axs[2,0].set_xlabel('')
axs[3,0].set_ylabel('')
axs[3,0].set_xlabel('Permutation importance, percentage points')
axs[0,1].set_ylabel('')
axs[0,1].set_xlabel('')
axs[1,1].set_ylabel('')
axs[1,1].set_xlabel('')
axs[2,1].set_ylabel('')
axs[2,1].set_xlabel('')
axs[3,1].set_ylabel('')
axs[3,1].set_xlabel('Permutation importance, percentage points')
plt.tight_layout()
plt.subplots_adjust(wspace=0.4,hspace=0.2)
plt.show()
